@@ -38,14 +38,16 @@
 
 #include "applier.h"
 
-#define MONITOR_CONTENTS_X 20
-#define MONITOR_CONTENTS_Y 10
-#define MONITOR_CONTENTS_WIDTH 157
-#define MONITOR_CONTENTS_HEIGHT 111
+#define MONITOR_CONTENTS_X 0 
+#define MONITOR_CONTENTS_Y 0
+#define MONITOR_CONTENTS_DEFAULT_WIDTH 64
+#define MONITOR_CONTENTS_DEFAULT_HEIGHT 48
 
 enum {
 	PROP_0,
-	PROP_TYPE
+	PROP_TYPE,
+	PROP_PREVIEW_WIDTH,
+	PROP_PREVIEW_HEIGHT
 };
 
 struct _BGApplierPrivate 
@@ -60,10 +62,6 @@ struct _BGApplierPrivate
 
 	GdkPixbuf          *wallpaper_pixbuf;      /* The "raw" wallpaper pixbuf */
 
-	gboolean            nautilus_running;      /* TRUE iff nautilus is
-						    * running, in which case we
-						    * block the renderer */
-
 	BGApplierType         type;                  /* Whether we render to the
 						    * root or the preview */
 
@@ -129,7 +127,9 @@ static void bg_applier_finalize      (GObject           *object);
 
 static void run_render_pipeline      (BGApplier           *bg_applier, 
 				      const BGPreferences *prefs);
-static void draw_disabled_message    (GtkWidget         *widget);
+static void draw_disabled_message    (GtkWidget         *widget,
+				      const guint width,
+				      const guint height);
 
 static void render_background        (BGApplier           *bg_applier,
 				      const BGPreferences *prefs);
@@ -178,10 +178,13 @@ static gboolean is_nautilus_running  (void);
 
 static gboolean cleanup_cb           (BGApplier *bg_applier);
 
-guint
+static void preview_realized_cb      (GtkWidget *preview,
+				      BGApplier *bg_applier);
+
+GType
 bg_applier_get_type (void)
 {
-	static guint bg_applier_type = 0;
+	static GType bg_applier_type = 0;
 
 	if (!bg_applier_type) {
 		static GTypeInfo bg_applier_info = {
@@ -213,8 +216,9 @@ bg_applier_init (BGApplier *bg_applier, BGApplierClass *class)
 	bg_applier->p->last_prefs       = NULL;
 	bg_applier->p->pixbuf           = NULL;
 	bg_applier->p->wallpaper_pixbuf = NULL;
-	bg_applier->p->nautilus_running = is_nautilus_running ();
 	bg_applier->p->timeout          = 0;
+	bg_applier->p->render_geom.width = -1;
+	bg_applier->p->render_geom.height = -1;
 }
 
 static void
@@ -237,6 +241,22 @@ bg_applier_class_init (BGApplierClass *class)
 				   0, 1, 0,
 				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_WIDTH,
+		 g_param_spec_uint ("preview_width",
+			 	    _("Preview Width"),
+				    _("Width if applier is a preview: Defaults to 64."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_WIDTH,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_HEIGHT,
+		 g_param_spec_uint ("preview_height",
+				    _("Preview Height"),
+				    _("Height if applier is a preview: Defaults to 48."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_HEIGHT,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	parent_class = 
 		G_OBJECT_CLASS (g_type_class_ref (G_TYPE_OBJECT));
 }
@@ -273,8 +293,13 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 		case BG_APPLIER_PREVIEW:
 			bg_applier->p->render_geom.x = MONITOR_CONTENTS_X;
 			bg_applier->p->render_geom.y = MONITOR_CONTENTS_Y;
-			bg_applier->p->render_geom.width = MONITOR_CONTENTS_WIDTH;
-			bg_applier->p->render_geom.height = MONITOR_CONTENTS_HEIGHT;
+			
+			if (bg_applier->p->render_geom.width == -1)
+			{
+				bg_applier->p->render_geom.width = MONITOR_CONTENTS_DEFAULT_WIDTH;
+				bg_applier->p->render_geom.height = MONITOR_CONTENTS_DEFAULT_HEIGHT;
+			}
+
 			break;
 
 		default:
@@ -284,6 +309,16 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 
 		break;
 
+	case PROP_PREVIEW_WIDTH:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.width = g_value_get_uint (value);
+		break;
+
+	case PROP_PREVIEW_HEIGHT:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.height = g_value_get_uint (value);
+		break;
+		
 	default:
 		g_warning ("Bad property set");
 		break;
@@ -359,6 +394,22 @@ bg_applier_new (BGApplierType type)
 	return object;
 }
 
+GObject *
+bg_applier_new_at_size (BGApplierType type,
+			const guint width,
+			const guint height)
+{
+	GObject *object;
+
+	object = g_object_new (bg_applier_get_type (),
+			       "type", type,
+			       "preview_width", width,
+			       "preview_height", height,
+			       NULL);
+
+	return object;
+}
+
 void
 bg_applier_apply_prefs (BGApplier           *bg_applier, 
 			const BGPreferences *prefs)
@@ -370,12 +421,20 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 
 	new_prefs = BG_PREFERENCES (bg_preferences_clone (prefs));
 
-	if (bg_applier->p->type == BG_APPLIER_ROOT && bg_applier->p->nautilus_running)
+	if (new_prefs->wallpaper_type == WPTYPE_NONE)
+	{
+		new_prefs->wallpaper_enabled = FALSE;
+		new_prefs->wallpaper_type = WPTYPE_CENTERED;
+	}
+
+	if (bg_applier->p->type == BG_APPLIER_ROOT && is_nautilus_running ()) {
 		set_root_pixmap ((GdkPixmap *) -1);
+		return;
+	}
 
 	if (!new_prefs->enabled) {
 		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
-			draw_disabled_message (bg_applier_get_preview_widget (bg_applier));
+			draw_disabled_message (bg_applier_get_preview_widget (bg_applier), bg_applier->p->render_geom.width, bg_applier->p->render_geom.height);
 		return;
 	}
 
@@ -392,8 +451,6 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 				gdk_pixbuf_new_from_file (new_prefs->wallpaper_filename, NULL);
 
 			if (bg_applier->p->wallpaper_pixbuf == NULL) {
-				g_warning (_("Could not load pixbuf \"%s\"; disabling wallpaper."),
-					   new_prefs->wallpaper_filename);
 				new_prefs->wallpaper_enabled = FALSE;
 			}
 			else if (bg_applier->p->type == BG_APPLIER_ROOT) {
@@ -432,64 +489,23 @@ bg_applier_render_color_p (const BGApplier *bg_applier, const BGPreferences *pre
 GtkWidget *
 bg_applier_get_preview_widget (BGApplier *bg_applier) 
 {
-	GdkPixbuf *pixbuf;
-	GdkPixmap *pixmap;
-	GdkBitmap *mask;
-	GdkVisual *visual;
-	GdkColormap *colormap;
-	gchar *filename;
-	GdkGC *gc;
+	if (bg_applier->p->preview_widget == NULL)
+	{
+		GdkPixmap *pixmap;
 
-	g_return_val_if_fail (bg_applier != NULL, NULL);
-	g_return_val_if_fail (IS_BG_APPLIER (bg_applier), NULL);
+		pixmap = gdk_pixmap_new (gdk_get_default_root_window (),
+					 bg_applier->p->render_geom.width,
+					 bg_applier->p->render_geom.height,
+					 -1);
+		bg_applier->p->preview_widget = gtk_image_new_from_pixmap (pixmap, NULL);
 
-	if (bg_applier->p->type != BG_APPLIER_PREVIEW)
-		return NULL;
-
-	if (bg_applier->p->preview_widget != NULL)
-		return bg_applier->p->preview_widget;
-
-	filename = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "monitor.png", TRUE, NULL);
-	visual = gdk_drawable_get_visual (gdk_get_default_root_window ());
-	colormap = gdk_drawable_get_colormap (gdk_get_default_root_window ());
-
-	gtk_widget_push_colormap (colormap);
-
-	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-
-	if (pixbuf == NULL) return NULL;
-
-	pixmap = gdk_pixmap_new (gdk_get_default_root_window (),
-				 gdk_pixbuf_get_width (pixbuf),
-				 gdk_pixbuf_get_height (pixbuf),
-				 visual->depth);
-	mask = gdk_pixmap_new (gdk_get_default_root_window (),
-			       gdk_pixbuf_get_width (pixbuf),
-			       gdk_pixbuf_get_height (pixbuf),
-			       1);
-
-	gc = gdk_gc_new (gdk_get_default_root_window ());
-
-	gdk_pixbuf_render_threshold_alpha (pixbuf, mask,
-					   0, 0, 0, 0,
-					   gdk_pixbuf_get_width (pixbuf),
-					   gdk_pixbuf_get_height (pixbuf),
-					   1);
-
-	gdk_gc_set_clip_mask (gc, mask);
-
-	gdk_pixbuf_render_to_drawable (pixbuf, pixmap, gc,
-				       0, 0, 0, 0,
-				       gdk_pixbuf_get_width (pixbuf),
-				       gdk_pixbuf_get_height (pixbuf),
-				       GDK_RGB_DITHER_MAX, 0, 0);
-
-	bg_applier->p->preview_widget = gtk_image_new_from_pixmap (pixmap, mask);
-	gtk_widget_show (bg_applier->p->preview_widget);
-	g_object_unref (G_OBJECT (pixbuf));
-	g_free (filename);
-
-	gtk_widget_pop_colormap ();
+		/* We need to initialize the pixmap, but this
+		 * needs GCs, so we have to wait until realize. */
+		g_signal_connect (G_OBJECT (bg_applier->p->preview_widget),
+				  "realize",
+				  (GCallback) preview_realized_cb,
+				  bg_applier);
+	}
 
 	return bg_applier->p->preview_widget;
 }
@@ -504,21 +520,19 @@ bg_applier_get_wallpaper_pixbuf (BGApplier *bg_applier)
 }
 
 static void
-draw_disabled_message (GtkWidget *widget)
+draw_disabled_message (GtkWidget *widget, const guint w, const guint h)
 {
 	GdkPixmap      *pixmap;
 	GdkColor        color;
 	PangoLayout    *layout;
 	PangoRectangle  extents;
 	GdkGC          *gc;
-	gint            x, y, w, h;
+	gint            x, y;
 	const char     *disabled_string = _("Disabled");
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_IMAGE (widget));
 
-	w = MONITOR_CONTENTS_WIDTH;
-	h = MONITOR_CONTENTS_HEIGHT;
 	x = MONITOR_CONTENTS_X;
 	y = MONITOR_CONTENTS_Y;
 
@@ -824,8 +838,8 @@ render_to_screen (BGApplier *bg_applier, const BGPreferences *prefs)
 						    bg_applier->p->render_geom.height);
 			}
 			else if (bg_applier->p->type == BG_APPLIER_ROOT) {
-				gdk_window_set_back_pixmap (GDK_ROOT_PARENT (), NULL, FALSE);
-				gdk_window_set_background (GDK_ROOT_PARENT (), prefs->color1);
+				gdk_window_set_back_pixmap (gdk_get_default_root_window (), NULL, FALSE);
+				gdk_window_set_background (gdk_get_default_root_window (), prefs->color1);
 			}
 		}
 	}
@@ -876,7 +890,8 @@ create_pixmap (BGApplier *bg_applier, const BGPreferences *prefs)
 #if 0
 		bg_applier->p->pixmap = GTK_PIXMAP (bg_applier->p->preview_widget)->pixmap;
 #else
-		gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
+		if (!bg_applier->p->pixmap)
+			gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
 #endif
 		bg_applier->p->pixmap_is_set = TRUE;
 		break;
@@ -1260,7 +1275,9 @@ need_wallpaper_load_p (const BGApplier *bg_applier, const BGPreferences *prefs)
 static gboolean
 need_root_pixmap_p (const BGApplier *bg_applier, const BGPreferences *prefs) 
 {
-	if (prefs->wallpaper_enabled == FALSE && prefs->gradient_enabled == FALSE)
+	if (bg_applier->p->pixmap == NULL)
+		return TRUE;
+	else if (prefs->wallpaper_enabled == FALSE && prefs->gradient_enabled == FALSE)
 		return FALSE;
 	else if (bg_applier->p->last_prefs == NULL)
 		return TRUE;
@@ -1348,7 +1365,7 @@ make_root_pixmap (gint width, gint height)
 
 	XCloseDisplay (display);
 
-	return gdk_pixmap_foreign_new ((GdkNativeWindow)xpixmap);
+	return gdk_pixmap_foreign_new (xpixmap);
 }
 
 /* Set the root pixmap, and properties pointing to it. We
@@ -1502,3 +1519,24 @@ cleanup_cb (BGApplier *bg_applier)
 	
 	return FALSE;
 }
+
+static void
+preview_realized_cb (GtkWidget *preview, BGApplier *bg_applier)
+{
+	GdkPixmap *pixmap;
+
+	/* Only draw clean image if no pref set yet */
+	if (bg_applier->p->last_prefs)
+		return;
+
+	gtk_image_get_pixmap (GTK_IMAGE (preview), &pixmap, NULL);
+	
+	gdk_draw_rectangle (pixmap,
+			    preview->style->bg_gc[preview->state],
+			    TRUE,
+			    bg_applier->p->render_geom.x,
+			    bg_applier->p->render_geom.y,
+			    bg_applier->p->render_geom.width,
+			    bg_applier->p->render_geom.height);
+}
+
