@@ -22,6 +22,7 @@
 
    Authors: John Sullivan <sullivan@eazel.com>
             Anders Carlsson <andersca@gnu.org>
+	    David Emory Watson <dwatson@cs.ucr.edu>
 */
 
 #include <config.h>
@@ -30,6 +31,7 @@
 #include "fm-error-reporting.h"
 #include "fm-list-model.h"
 #include <eel/eel-cell-renderer-pixbuf-list.h>
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtktreeselection.h>
@@ -46,7 +48,14 @@ struct FMListViewDetails {
 	FMListModel *model;
 
 	GtkTreeViewColumn   *file_name_column;
-	GtkCellRendererText *file_name_cell;
+
+	GtkCellRendererPixbuf *pixbuf_cell;
+	GtkCellRendererText   *file_name_cell;
+	GtkCellRendererText   *size_cell;
+	GtkCellRendererText   *type_cell;
+	GtkCellRendererText   *date_modified_cell;
+
+	NautilusZoomLevel zoom_level;
 };
 
 static GtkTargetEntry drag_types [] = {
@@ -62,10 +71,15 @@ static GtkTargetEntry drag_types [] = {
  */
 #define LIST_VIEW_MINIMUM_ROW_HEIGHT	28
 
+static int                      click_policy_auto_value;
 static NautilusFileSortType	default_sort_order_auto_value;
 static gboolean			default_sort_reversed_auto_value;
+static NautilusZoomLevel        default_zoom_level_auto_value;
 
-static GList *     fm_list_view_get_selection         (FMDirectoryView *view);
+static GList *              fm_list_view_get_selection         (FMDirectoryView *view);
+static void                 fm_list_view_set_zoom_level        (FMListView *view,
+								NautilusZoomLevel new_level,
+								gboolean always_set_level);
 
 GNOME_CLASS_BOILERPLATE (FMListView, fm_list_view,
 			 FMDirectoryView, FM_TYPE_DIRECTORY_VIEW)
@@ -88,7 +102,7 @@ list_activate_callback (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewCo
 	
 	view = FM_DIRECTORY_VIEW (user_data);
 
-	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_DOUBLE) {
+	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE) {
 		file_list = fm_list_view_get_selection (view);
 		fm_directory_view_activate_files (view, file_list);
 		nautilus_file_list_free (file_list);
@@ -128,25 +142,17 @@ static void
 event_after_callback (GtkWidget *widget, GdkEventAny *event, gpointer callback_data)
 {
 	FMDirectoryView *view;
-	GList *file_list;
 
 	view = FM_DIRECTORY_VIEW (callback_data);
 
 	if (event->type == GDK_BUTTON_PRESS
-	    && event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))) {
-		if (((GdkEventButton *) event)->button == 1
-		    && eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
-			/* Handle single click activation preference. */
-			file_list = fm_list_view_get_selection (view);
-			fm_directory_view_activate_files (view, file_list);
-			nautilus_file_list_free (file_list);
-		} else if (((GdkEventButton *) event)->button == 3) {
-			/* Put up the right kind of menu if we right click in the tree view. */
-			if (tree_view_has_selection (GTK_TREE_VIEW (widget))) {
-				fm_directory_view_pop_up_selection_context_menu (view, (GdkEventButton *) event);
-			} else {
-				fm_directory_view_pop_up_background_context_menu (view, (GdkEventButton *) event);
-			}
+	    && event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
+	    && (((GdkEventButton *) event)->button == 3)) {
+		/* Put up the right kind of menu if we right click in the tree view. */
+		if (tree_view_has_selection (GTK_TREE_VIEW (widget))) {
+			fm_directory_view_pop_up_selection_context_menu (view, (GdkEventButton *) event);
+		} else {
+			fm_directory_view_pop_up_background_context_menu (view, (GdkEventButton *) event);
 		}
 	}
 }
@@ -154,17 +160,107 @@ event_after_callback (GtkWidget *widget, GdkEventAny *event, gpointer callback_d
 static gboolean
 button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callback_data)
 {
-	/* Deselect if people click outside any row. */
-	if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget))
-	    && !gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget),
-					       event->x, event->y, NULL, NULL, NULL, NULL)) {
-		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)));
+	GtkTreeView *tree_view;
+	GtkTreePath *path;
+
+	tree_view = GTK_TREE_VIEW (widget);
+
+	if (event->window != gtk_tree_view_get_bin_window (tree_view)) {
+		return FALSE;
+	}
+
+	if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
+					   &path, NULL, NULL, NULL)) {
+		if (event->button == 3
+		    && gtk_tree_selection_path_is_selected (gtk_tree_view_get_selection (tree_view), path)) {
+			/* Don't let the default code run because if multiple rows
+			   are selected it will unselect all but one row; but we
+			   want the right click menu to apply to everything that's
+			   currently selected. */
+			return TRUE;
+		}
+
+		gtk_tree_path_free (path);
+	} else {
+		/* Deselect if people click outside any row. It's OK to
+		   let default code run; it won't reselect anything. */
+		gtk_tree_selection_unselect_all (gtk_tree_view_get_selection (tree_view));
 	}
 
-	/* Let the default code run in any case; it won't reselect anything. */
 	return FALSE;
 }
 
+static gboolean
+button_release_callback (GtkWidget *widget, GdkEventButton *event, gpointer callback_data)
+{
+	FMDirectoryView *view;
+	GList *file_list;
+
+	view = FM_DIRECTORY_VIEW (callback_data);
+
+	if (event->window == gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget)) &&
+	    event->button == 1 &&
+	    click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
+		/* Handle single click activation preference. */
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+	}
+
+	return FALSE;
+}
+
+static gboolean
+key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_data)
+{
+	FMDirectoryView *view;
+	GList *file_list;
+
+	view = FM_DIRECTORY_VIEW (callback_data);
+
+	switch (event->keyval) {
+	case GDK_Return:
+	case GDK_KP_Enter:
+		file_list = fm_list_view_get_selection (view);
+		fm_directory_view_activate_files (view, file_list);
+		nautilus_file_list_free (file_list);
+		return TRUE;
+
+	default:
+		break;
+	}
+
+	return FALSE;
+}
+
+static void
+sort_column_changed_callback (GtkTreeSortable *sortable, 
+			      FMListView *view)
+{
+	NautilusFile *file;
+	gint sort_column_id;
+	GtkSortType reversed;
+	char *sort_attr, *default_sort_attr;
+	char *reversed_attr, *default_reversed_attr;
+
+	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
+
+	gtk_tree_sortable_get_sort_column_id (sortable, &sort_column_id, &reversed);
+
+	sort_attr = fm_list_model_get_attribute_from_sort_column_id (sort_column_id);
+	sort_column_id = fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value);
+	default_sort_attr = fm_list_model_get_attribute_from_sort_column_id (sort_column_id);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN,
+				    default_sort_attr, sort_attr);
+	g_free (default_sort_attr);
+	g_free (sort_attr);
+
+	default_reversed_attr = (default_sort_reversed_auto_value ? "true" : "false");
+	reversed_attr = (reversed ? "true" : "false");
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED,
+				    default_reversed_attr, reversed_attr);
+}
+
 static void
 cell_renderer_edited (GtkCellRendererText *cell,
 		      const char          *path_str,
@@ -192,24 +288,6 @@ cell_renderer_edited (GtkCellRendererText *cell,
 	nautilus_file_unref (file);
 }
 
-static void
-click_policy_changed (gpointer callback_data)
-{
-	FMListView *view;
-
-	view = FM_LIST_VIEW (callback_data);
-
-	if (eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY) == NAUTILUS_CLICK_POLICY_SINGLE) {
-		g_object_set (G_OBJECT (view->details->file_name_cell),
-			      "underline", PANGO_UNDERLINE_SINGLE,
-			      NULL);
-	} else {
-		g_object_set (G_OBJECT (view->details->file_name_cell),
-			      "underline", PANGO_UNDERLINE_NONE,
-			      NULL);
-	}
-}
-
 static void
 create_and_set_up_tree_view (FMListView *view)
 {
@@ -217,6 +295,7 @@ create_and_set_up_tree_view (FMListView *view)
 	GtkTreeViewColumn *column;
 	
 	view->details->tree_view = GTK_TREE_VIEW (gtk_tree_view_new ());
+
 	g_signal_connect_object (gtk_tree_view_get_selection (view->details->tree_view),
 				 "changed",
 				 G_CALLBACK (list_selection_changed_callback), view, 0);
@@ -227,19 +306,27 @@ create_and_set_up_tree_view (FMListView *view)
 				 G_CALLBACK (event_after_callback), view, 0);
 	g_signal_connect_object (view->details->tree_view, "button_press_event",
 				 G_CALLBACK (button_press_callback), view, 0);
-
+	g_signal_connect_object (view->details->tree_view, "button_release_event",
+				 G_CALLBACK (button_release_callback), view, 0);
+	g_signal_connect_object (view->details->tree_view, "key_press_event",
+				 G_CALLBACK (key_press_callback), view, 0);
+	
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
-	g_object_unref (view->details->model);
-	
+
+	g_signal_connect_object (view->details->model, "sort_column_changed",
+				 G_CALLBACK (sort_column_changed_callback), view, 0);
+
 	gtk_tree_selection_set_mode (gtk_tree_view_get_selection (view->details->tree_view), GTK_SELECTION_MULTIPLE);
 	gtk_tree_view_set_rules_hint (view->details->tree_view, TRUE);
 
-	gtk_tree_view_enable_model_drag_source (view->details->tree_view, 0, drag_types, G_N_ELEMENTS (drag_types),
+	gtk_tree_view_enable_model_drag_source (view->details->tree_view, 0,
+						drag_types, G_N_ELEMENTS (drag_types),
 						GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
-	
+
 	/* Create the file name column */
 	cell = gtk_cell_renderer_pixbuf_new ();
+	view->details->pixbuf_cell = (GtkCellRendererPixbuf *)cell;
 	
 	view->details->file_name_column = gtk_tree_view_column_new ();
 	gtk_tree_view_column_set_sort_column_id (view->details->file_name_column, FM_LIST_MODEL_NAME_COLUMN);
@@ -247,11 +334,9 @@ create_and_set_up_tree_view (FMListView *view)
 	gtk_tree_view_column_set_resizable (view->details->file_name_column, TRUE);
 
 	gtk_tree_view_column_pack_start (view->details->file_name_column, cell, FALSE);
-	gtk_tree_view_column_set_attributes (view->details->file_name_column, cell,
-					     "pixbuf", FM_LIST_MODEL_ICON_COLUMN,
-					     NULL);
 
 	cell = gtk_cell_renderer_text_new ();
+	view->details->file_name_cell = (GtkCellRendererText *)cell;
 	g_signal_connect (cell, "edited", G_CALLBACK (cell_renderer_edited), view);
 
 	gtk_tree_view_column_pack_start (view->details->file_name_column, cell, TRUE);
@@ -261,14 +346,9 @@ create_and_set_up_tree_view (FMListView *view)
 					     NULL);
 	gtk_tree_view_append_column (view->details->tree_view, view->details->file_name_column);
 
-	/* Set up underline of file name. */
-	view->details->file_name_cell = (GtkCellRendererText *)cell;
-	eel_preferences_add_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
-				      click_policy_changed, view);
-	click_policy_changed (view);
-
 	/* Create the size column */
 	cell = gtk_cell_renderer_text_new ();
+	view->details->size_cell = (GtkCellRendererText *)cell;
 	g_object_set (G_OBJECT (cell),
 		      "xalign", 1.0,
 		      NULL);
@@ -282,6 +362,7 @@ create_and_set_up_tree_view (FMListView *view)
 
 	/* Create the type column */
 	cell = gtk_cell_renderer_text_new ();
+	view->details->type_cell = (GtkCellRendererText *)cell;
 	column = gtk_tree_view_column_new_with_attributes (_("Type"),
 							   cell,
 							   "text", FM_LIST_MODEL_TYPE_COLUMN,
@@ -292,6 +373,7 @@ create_and_set_up_tree_view (FMListView *view)
 
 	/* Create the date modified column */
 	cell = gtk_cell_renderer_text_new ();
+	view->details->date_modified_cell = (GtkCellRendererText *)cell;
 	column = gtk_tree_view_column_new_with_attributes (_("Date Modified"),
 							   cell,
 							   "text", FM_LIST_MODEL_DATE_MODIFIED_COLUMN,
@@ -338,6 +420,21 @@ set_sort_order_from_metadata_and_preferences (FMListView *list_view)
 					      sort_reversed ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
 }
 
+static void
+set_zoom_level_from_metadata_and_preferences (FMListView *list_view)
+{
+	NautilusFile *file;
+	int level;
+
+	if (fm_directory_view_supports_zooming (FM_DIRECTORY_VIEW (list_view))) {
+		file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (list_view));
+		level = nautilus_file_get_integer_metadata (file,
+							    NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, 
+							    default_zoom_level_auto_value);
+		fm_list_view_set_zoom_level (list_view, level, TRUE);
+	}
+}
+
 static void
 fm_list_view_begin_loading (FMDirectoryView *view)
 {
@@ -349,6 +446,7 @@ fm_list_view_begin_loading (FMDirectoryView *view)
 						      fm_directory_view_get_directory_as_file (view));
 
 	set_sort_order_from_metadata_and_preferences (list_view);
+	set_zoom_level_from_metadata_and_preferences (list_view);
 }
 
 static void
@@ -358,7 +456,9 @@ fm_list_view_clear (FMDirectoryView *view)
 
 	list_view = FM_LIST_VIEW (view);
 	
-	fm_list_model_clear (list_view->details->model);
+	if (list_view->details->model != NULL) {
+		fm_list_model_clear (list_view->details->model);
+	}
 }
 
 static void
@@ -448,14 +548,138 @@ fm_list_view_reset_to_defaults (FMDirectoryView *view)
 	NautilusFile *file;
 
 	file = fm_directory_view_get_directory_as_file (view);
+
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN, NULL, NULL);
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED, NULL, NULL);
+	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, NULL, NULL);
 
 	gtk_tree_sortable_set_sort_column_id
 		(GTK_TREE_SORTABLE (FM_LIST_VIEW (view)->details->model),
 		 fm_list_model_get_sort_column_id_from_sort_type (default_sort_order_auto_value),
 		 default_sort_reversed_auto_value ? GTK_SORT_DESCENDING : GTK_SORT_ASCENDING);
-        fm_directory_view_restore_default_zoom_level (view);
+
+	fm_list_view_set_zoom_level (FM_LIST_VIEW (view), default_zoom_level_auto_value, FALSE);
+}
+
+static void
+fm_list_view_set_zoom_level (FMListView *view,
+			     NautilusZoomLevel new_level,
+			     gboolean always_set_level)
+{
+	static double pango_scale[7] = { PANGO_SCALE_X_SMALL,
+					 PANGO_SCALE_SMALL,
+					 PANGO_SCALE_MEDIUM,
+					 PANGO_SCALE_LARGE,
+					 PANGO_SCALE_X_LARGE,
+					 PANGO_SCALE_XX_LARGE,
+					 1.2 * PANGO_SCALE_XX_LARGE };
+	int icon_size;
+	int column;
+
+	g_return_if_fail (FM_IS_LIST_VIEW (view));
+	g_return_if_fail (new_level >= NAUTILUS_ZOOM_LEVEL_SMALLEST &&
+			  new_level <= NAUTILUS_ZOOM_LEVEL_LARGEST);
+
+	if (view->details->zoom_level == new_level) {
+		if (always_set_level) {
+			fm_directory_view_set_zoom_level (FM_DIRECTORY_VIEW(view), new_level);
+		}
+		return;
+	}
+
+	view->details->zoom_level = new_level;
+	fm_directory_view_set_zoom_level (FM_DIRECTORY_VIEW(view), new_level);
+
+	nautilus_file_set_integer_metadata
+		(fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view)), 
+		 NAUTILUS_METADATA_KEY_LIST_VIEW_ZOOM_LEVEL, 
+		 default_zoom_level_auto_value,
+		 new_level);
+
+	/* Select correctly scaled icons. */
+	column = fm_list_model_get_column_id_from_zoom_level (new_level);
+	gtk_tree_view_column_set_attributes (view->details->file_name_column,
+					     GTK_CELL_RENDERER (view->details->pixbuf_cell),
+					     "pixbuf", column,
+					     NULL);
+
+	/* Scale text. */
+	g_object_set (G_OBJECT (view->details->file_name_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->size_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->type_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+	g_object_set (G_OBJECT (view->details->date_modified_cell),
+		      "scale", pango_scale[new_level],
+		      NULL);
+
+	/* Make all rows the same size. */
+	icon_size = nautilus_get_icon_size_for_zoom_level (new_level);
+	gtk_cell_renderer_set_fixed_size (GTK_CELL_RENDERER (view->details->pixbuf_cell),
+					  -1, icon_size);
+
+	fm_directory_view_update_menus (FM_DIRECTORY_VIEW (view));
+}
+
+static void
+fm_list_view_bump_zoom_level (FMDirectoryView *view, int zoom_increment)
+{
+	FMListView *list_view;
+	NautilusZoomLevel new_level;
+
+	g_return_if_fail (FM_IS_LIST_VIEW (view));
+
+	list_view = FM_LIST_VIEW (view);
+	new_level = list_view->details->zoom_level + zoom_increment;
+
+	if (new_level >= NAUTILUS_ZOOM_LEVEL_SMALLEST &&
+	    new_level <= NAUTILUS_ZOOM_LEVEL_LARGEST) {
+		fm_list_view_set_zoom_level (list_view, new_level, FALSE);
+	}
+}
+
+static void
+fm_list_view_zoom_to_level (FMDirectoryView *view, int zoom_level)
+{
+	FMListView *list_view;
+
+	g_return_if_fail (FM_IS_LIST_VIEW (view));
+
+	list_view = FM_LIST_VIEW (view);
+
+	fm_list_view_set_zoom_level (list_view, zoom_level, FALSE);
+}
+
+static void
+fm_list_view_restore_default_zoom_level (FMDirectoryView *view)
+{
+	FMListView *list_view;
+
+	g_return_if_fail (FM_IS_LIST_VIEW (view));
+
+	list_view = FM_LIST_VIEW (view);
+
+	fm_list_view_set_zoom_level (list_view, default_zoom_level_auto_value, FALSE);
+}
+
+static gboolean 
+fm_list_view_can_zoom_in (FMDirectoryView *view) 
+{
+	g_return_val_if_fail (FM_IS_LIST_VIEW (view), FALSE);
+
+	return FM_LIST_VIEW (view)->details->zoom_level	< NAUTILUS_ZOOM_LEVEL_LARGEST;
+}
+
+static gboolean 
+fm_list_view_can_zoom_out (FMDirectoryView *view) 
+{
+	g_return_val_if_fail (FM_IS_LIST_VIEW (view), FALSE);
+
+	return FM_LIST_VIEW (view)->details->zoom_level > NAUTILUS_ZOOM_LEVEL_SMALLEST;
 }
 
 static void
@@ -479,41 +703,43 @@ fm_list_view_start_renaming_file (FMDirectoryView *view, NautilusFile *file)
 				  TRUE);
 }
 
-
-#if GNOME2_CONVERSION_COMPLETE
-
-/* This is needed when writing out the sort to metadata. But we don't
- * need this list here. We can probably do this using the table inside
- * FMListModel.
- */
-
-static char *
-get_attribute_from_sort_type (NautilusFileSortType sort_type)
+static void
+click_policy_changed_callback (gpointer callback_data)
 {
-	switch (sort_type) {
-	case NAUTILUS_FILE_SORT_BY_DISPLAY_NAME:
-		return g_strdup ("name");
-	case NAUTILUS_FILE_SORT_BY_SIZE:
-		return g_strdup ("size");
-	case NAUTILUS_FILE_SORT_BY_TYPE:
-		return g_strdup ("type");
-	case NAUTILUS_FILE_SORT_BY_MTIME:
-		return g_strdup ("date_modified");
-	case NAUTILUS_FILE_SORT_BY_EMBLEMS:
-		return g_strdup ("emblems");
-	default:
-		g_warning ("unknown sort type %d in get_attribute_from_sort_type", sort_type);
-		return g_strdup ("name");
+	FMListView *view;
+
+	view = FM_LIST_VIEW (callback_data);
+
+	if (click_policy_auto_value == NAUTILUS_CLICK_POLICY_SINGLE) {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_SINGLE,
+			      NULL);
+	} else {
+		g_object_set (G_OBJECT (view->details->file_name_cell),
+			      "underline", PANGO_UNDERLINE_NONE,
+			      NULL);
 	}
 }
 
-static char *
-real_get_default_sort_attribute (FMListView *view)
+static void
+default_sort_order_changed_callback (gpointer callback_data)
 {
-	return get_attribute_from_sort_type (default_sort_order_auto_value);
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (callback_data);
+
+	set_sort_order_from_metadata_and_preferences (list_view);
 }
 
-#endif
+static void
+default_zoom_level_changed_callback (gpointer callback_data)
+{
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (callback_data);
+
+	set_zoom_level_from_metadata_and_preferences (list_view);
+}
 
 static void
 fm_list_view_sort_directories_first_changed (FMDirectoryView *view)
@@ -527,14 +753,26 @@ fm_list_view_sort_directories_first_changed (FMDirectoryView *view)
 }
 
 static void
-fm_list_view_finalize (GObject *object)
+fm_list_view_dispose (GObject *object)
 {
 	FMListView *list_view;
 
 	list_view = FM_LIST_VIEW (object);
 
-	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
-					 click_policy_changed, list_view);
+	if (list_view->details->model) {
+		g_object_unref (list_view->details->model);
+		list_view->details->model = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_class)->dispose (object);
+}
+
+static void
+fm_list_view_finalize (GObject *object)
+{
+	FMListView *list_view;
+
+	list_view = FM_LIST_VIEW (object);
 
 	g_free (list_view->details);
 
@@ -553,7 +791,6 @@ fm_list_view_emblems_changed (FMDirectoryView *directory_view)
 #endif
 }
 
-
 static void
 fm_list_view_class_init (FMListViewClass *class)
 {
@@ -561,24 +798,22 @@ fm_list_view_class_init (FMListViewClass *class)
 
 	fm_directory_view_class = FM_DIRECTORY_VIEW_CLASS (class);
 
+	G_OBJECT_CLASS (class)->dispose = fm_list_view_dispose;
 	G_OBJECT_CLASS (class)->finalize = fm_list_view_finalize;
 
 	fm_directory_view_class->add_file = fm_list_view_add_file;
 	fm_directory_view_class->begin_loading = fm_list_view_begin_loading;
-#if GNOME2_CONVERSION_COMPLETE
-	fm_directory_view_class->bump_zoom_level = xxx;
-	fm_directory_view_class->can_zoom_in = xxx;
-	fm_directory_view_class->can_zoom_out = xxx;
-#endif
+	fm_directory_view_class->bump_zoom_level = fm_list_view_bump_zoom_level;
+	fm_directory_view_class->zoom_to_level = fm_list_view_zoom_to_level;
+	fm_directory_view_class->can_zoom_in = fm_list_view_can_zoom_in;
+	fm_directory_view_class->can_zoom_out = fm_list_view_can_zoom_out;
 	fm_directory_view_class->clear = fm_list_view_clear;
 	fm_directory_view_class->file_changed = fm_list_view_file_changed;
 	fm_directory_view_class->get_background_widget = fm_list_view_get_background_widget;
 	fm_directory_view_class->get_selection = fm_list_view_get_selection;
 	fm_directory_view_class->is_empty = fm_list_view_is_empty;
 	fm_directory_view_class->reset_to_defaults = fm_list_view_reset_to_defaults;
-#if GNOME2_CONVERSION_COMPLETE
-	fm_directory_view_class->restore_default_zoom_level = xxx;
-#endif
+	fm_directory_view_class->restore_default_zoom_level = fm_list_view_restore_default_zoom_level;
 	fm_directory_view_class->remove_file = fm_list_view_remove_file;
 	fm_directory_view_class->select_all = fm_list_view_select_all;
 	fm_directory_view_class->set_selection = fm_list_view_set_selection;
@@ -586,10 +821,14 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->sort_directories_first_changed = fm_list_view_sort_directories_first_changed;
 	fm_directory_view_class->start_renaming_file = fm_list_view_start_renaming_file;
 
+	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_CLICK_POLICY,
+				       &click_policy_auto_value);
 	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
 				       (int *) &default_sort_order_auto_value);
 	eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
 					  &default_sort_reversed_auto_value);
+	eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
+				       (int *) &default_zoom_level_auto_value);
 }
 
 static void
@@ -599,5 +838,20 @@ fm_list_view_instance_init (FMListView *list_view)
 
 	create_and_set_up_tree_view (list_view);
 
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_CLICK_POLICY,
+						  click_policy_changed_callback,
+						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_ORDER,
+						  default_sort_order_changed_callback,
+						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
+						  default_sort_order_changed_callback,
+						  list_view, G_OBJECT (list_view));
+	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_ZOOM_LEVEL,
+						  default_zoom_level_changed_callback,
+						  list_view, G_OBJECT (list_view));
+
+	click_policy_changed_callback (list_view);
+
 	fm_list_view_sort_directories_first_changed (FM_DIRECTORY_VIEW (list_view));
 }
