@@ -104,7 +104,6 @@ struct NautilusSidebarTabsDetails {
 	GdkColor prelit_text_color;
 
 	GdkPixbuf *tab_piece_images[LAST_TAB_OFFSET];
-	PangoFontDescription *tab_font;
 	int tab_height;
 	int tab_left_offset;
 	char *title;
@@ -138,7 +137,8 @@ static void     draw_or_layout_all_tabs                 (NautilusSidebarTabs
 							 gboolean                  layout_only);
 static TabItem* tab_item_find_by_name                   (NautilusSidebarTabs      *sidebar_tabs,
 							 const char               *name);
-static void     default_font_changed_callback           (gpointer                  callback_data);
+static void     style_set                               (GtkWidget                *widget,
+							 GtkStyle                 *previous_style);
 
 EEL_CLASS_BOILERPLATE (NautilusSidebarTabs, nautilus_sidebar_tabs, GTK_TYPE_WIDGET)
 
@@ -154,6 +154,7 @@ nautilus_sidebar_tabs_class_init (NautilusSidebarTabsClass *class)
 	object_class->destroy = nautilus_sidebar_tabs_destroy;
 	widget_class->expose_event = nautilus_sidebar_tabs_expose;
 	widget_class->size_allocate = nautilus_sidebar_tabs_size_allocate;
+	widget_class->style_set = style_set;
 }
 
 /* utilities to set up the text color alternatives */
@@ -235,35 +236,19 @@ nautilus_sidebar_tabs_load_theme_data (NautilusSidebarTabs *sidebar_tabs)
 				}
 			}
 		}	
-	}	
-
-	default_font_changed_callback (sidebar_tabs);
+	}
+	
+	gtk_widget_queue_resize (GTK_WIDGET (sidebar_tabs));
 }
 
-/* Use the font from preferences */
+static void recalculate_size(NautilusSidebarTabs *sidebar_tabs);
+
 static void
-default_font_changed_callback (gpointer callback_data)
+style_set (GtkWidget *widget,
+	   GtkStyle  *previous_style)
 {
-	PangoFontDescription *new_font;
-	char *font_name;
-	NautilusSidebarTabs *sidebar_tabs;
-
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR_TABS (callback_data));
-
-	sidebar_tabs = NAUTILUS_SIDEBAR_TABS (callback_data);
-
-	font_name = eel_preferences_get (NAUTILUS_PREFERENCES_DEFAULT_FONT);
-	new_font = pango_font_description_from_string (font_name);
-	pango_font_description_set_size (new_font, DEFAULT_FONT_SIZE * PANGO_SCALE);
-	
-	if (sidebar_tabs->details->tab_font != NULL) {
-		pango_font_description_free (sidebar_tabs->details->tab_font);
-		sidebar_tabs->details->tab_font = NULL;
-	}
-
-	sidebar_tabs->details->tab_font = new_font;
-
-	gtk_widget_queue_resize (GTK_WIDGET (sidebar_tabs));
+	recalculate_size (NAUTILUS_SIDEBAR_TABS (widget));
+	gtk_widget_queue_resize (widget);
 }
 
 /* initialize a newly allocated sidebar tabs object */
@@ -303,10 +288,6 @@ nautilus_sidebar_tabs_init (NautilusSidebarTabs *sidebar_tabs)
 				     (EelPreferencesCallback) nautilus_sidebar_tabs_load_theme_data, 
 				     sidebar_tabs);
 
-	eel_preferences_add_callback (NAUTILUS_PREFERENCES_DEFAULT_FONT,
-				      default_font_changed_callback,
-				      sidebar_tabs);
-	
 	sidebar_tabs->details->title_prelit = FALSE;
 }
 
@@ -406,12 +387,6 @@ nautilus_sidebar_tabs_destroy (GtkObject *object)
 						 (EelPreferencesCallback) nautilus_sidebar_tabs_load_theme_data, 
 						 sidebar_tabs);
 
-		eel_preferences_remove_callback (NAUTILUS_PREFERENCES_DEFAULT_FONT,
-						 default_font_changed_callback,
-						 sidebar_tabs);
-
-		pango_font_description_free (sidebar_tabs->details->tab_font);
-		
 		g_free (sidebar_tabs->details);
 		sidebar_tabs->details = NULL;
 	}
@@ -603,11 +578,18 @@ make_tab_text_layout (NautilusSidebarTabs *sidebar_tabs,
 		      const char *tab_name)
 {
 	PangoLayout *layout;
+	PangoFontDescription *desc;
+	PangoContext *context;
 
-	layout = pango_layout_new (eel_gtk_widget_get_pango_ft2_context (GTK_WIDGET (sidebar_tabs)));
+	context = eel_gtk_widget_get_pango_ft2_context (GTK_WIDGET (sidebar_tabs));
+	layout = pango_layout_new (context);
 	pango_layout_set_text (layout, tab_name, -1);
-	pango_layout_set_font_description (layout, sidebar_tabs->details->tab_font);
 	
+	desc = pango_font_description_copy (pango_context_get_font_description (context));
+	pango_font_description_set_size (desc, DEFAULT_FONT_SIZE * PANGO_SCALE);
+	pango_layout_set_font_description (layout, desc);
+	pango_font_description_free (desc);
+
 	return layout;
 }
 
