@@ -173,9 +173,8 @@ nautilus_window_instance_init (NautilusWindow *window)
 	window->details = g_new0 (NautilusWindowDetails, 1);
 
 	/* CORBA and Bonobo setup, which must be done before the location bar setup */
-	window->details->ui_container = bonobo_ui_container_new ();
-	bonobo_ui_container_set_engine (window->details->ui_container,
-					bonobo_window_get_ui_engine (BONOBO_WINDOW (window)));
+	window->details->ui_container = bonobo_window_get_ui_container (BONOBO_WINDOW (window));
+	bonobo_object_ref (window->details->ui_container);
 
 	window->details->tooltips = gtk_tooltips_new ();
 	g_object_ref (G_OBJECT (window->details->tooltips));
@@ -193,7 +192,7 @@ nautilus_window_instance_init (NautilusWindow *window)
 	/* Create a separate component so when we remove the status
 	 * we don't loose the status bar
 	 */
-      	window->details->status_ui = bonobo_ui_component_new ("Status Component");
+      	window->details->status_ui = bonobo_ui_component_new ("Status Component");  
 	bonobo_ui_component_set_container
 		(window->details->status_ui,
 		 nautilus_window_get_ui_container (window),
@@ -424,7 +423,7 @@ nautilus_window_zoom_in (NautilusWindow *window)
 }
 
 void
-nautilus_window_zoom_to_level (NautilusWindow *window, double level)
+nautilus_window_zoom_to_level (NautilusWindow *window, float level)
 {
 	if (window->content_view != NULL) {
 		nautilus_view_frame_set_zoom_level (window->content_view, level);
@@ -1893,6 +1892,7 @@ nautilus_window_hide_toolbar (NautilusWindow *window)
 void 
 nautilus_window_show_toolbar (NautilusWindow *window)
 {
+	nautilus_window_activate_throbber (window);
 	show_dock_item (window, TOOLBAR_PATH);
 }
 
